@@ -11,25 +11,24 @@ use log::{debug, error, warn};
 use rayon::prelude::*;
 use rustc_hash::FxHashMap;
 
-use ruff_diagnostics::Diagnostic;
-use ruff_linter::message::Message;
+use ruff_db::panic::catch_unwind;
+use ruff_linter::OldDiagnostic;
+use ruff_linter::package::PackageRoot;
 use ruff_linter::registry::Rule;
 use ruff_linter::settings::types::UnsafeFixes;
-use ruff_linter::settings::{flags, LinterSettings};
-use ruff_linter::{fs, warn_user_once, IOError};
+use ruff_linter::settings::{LinterSettings, flags};
+use ruff_linter::{IOError, fs, warn_user_once};
 use ruff_source_file::SourceFileBuilder;
-use ruff_text_size::{TextRange, TextSize};
+use ruff_text_size::TextRange;
 use ruff_workspace::resolver::{
-    match_exclusion, python_files_in_path, PyprojectConfig, ResolvedFile,
+    PyprojectConfig, ResolvedFile, match_exclusion, python_files_in_path,
 };
 
 use crate::args::ConfigArguments;
 use crate::cache::{Cache, PackageCacheMap, PackageCaches};
 use crate::diagnostics::Diagnostics;
-use crate::panic::catch_unwind;
 
 /// Run the linter over a collection of files.
-#[allow(clippy::too_many_arguments)]
 pub(crate) fn check(
     files: &[PathBuf],
     pyproject_config: &PyprojectConfig,
@@ -59,7 +58,7 @@ pub(crate) fn check(
     );
 
     // Load the caches.
-    let caches = if bool::from(cache) {
+    let caches = if cache.is_enabled() {
         Some(PackageCacheMap::init(&package_roots, &resolver))
     } else {
         None
@@ -87,7 +86,9 @@ pub(crate) fn check(
                     return None;
                 }
 
-                let cache_root = package.unwrap_or_else(|| path.parent().unwrap_or(path));
+                let cache_root = package
+                    .map(PackageRoot::path)
+                    .unwrap_or_else(|| path.parent().unwrap_or(path));
                 let cache = caches.get(cache_root);
 
                 lint_path(
@@ -128,10 +129,10 @@ pub(crate) fn check(
                         SourceFileBuilder::new(path.to_string_lossy().as_ref(), "").finish();
 
                     Diagnostics::new(
-                        vec![Message::from_diagnostic(
-                            Diagnostic::new(IOError { message }, TextRange::default()),
-                            dummy,
-                            TextSize::default(),
+                        vec![OldDiagnostic::new(
+                            IOError { message },
+                            TextRange::default(),
+                            &dummy,
                         )],
                         FxHashMap::default(),
                     )
@@ -165,23 +166,22 @@ pub(crate) fn check(
             |a, b| (a.0 + b.0, a.1 + b.1),
         );
 
-    all_diagnostics.messages.sort();
+    all_diagnostics.inner.sort();
 
     // Store the caches.
     caches.persist()?;
 
     let duration = start.elapsed();
-    debug!("Checked {:?} files in: {:?}", checked_files, duration);
+    debug!("Checked {checked_files:?} files in: {duration:?}");
 
     Ok(all_diagnostics)
 }
 
 /// Wraps [`lint_path`](crate::diagnostics::lint_path) in a [`catch_unwind`](std::panic::catch_unwind) and emits
 /// a diagnostic if the linting the file panics.
-#[allow(clippy::too_many_arguments)]
 fn lint_path(
     path: &Path,
-    package: Option<&Path>,
+    package: Option<PackageRoot<'_>>,
     settings: &LinterSettings,
     cache: Option<&Cache>,
     noqa: flags::Noqa,
@@ -227,9 +227,9 @@ mod test {
     use ruff_linter::message::{Emitter, EmitterContext, TextEmitter};
     use ruff_linter::registry::Rule;
     use ruff_linter::settings::types::UnsafeFixes;
-    use ruff_linter::settings::{flags, LinterSettings};
-    use ruff_workspace::resolver::{PyprojectConfig, PyprojectDiscoveryStrategy};
+    use ruff_linter::settings::{LinterSettings, flags};
     use ruff_workspace::Settings;
+    use ruff_workspace::resolver::{PyprojectConfig, PyprojectDiscoveryStrategy};
 
     use crate::args::ConfigArguments;
 
@@ -268,8 +268,7 @@ mod test {
 
         // Run
         let diagnostics = check(
-            // Notebooks are not included by default
-            &[tempdir.path().to_path_buf(), notebook],
+            &[tempdir.path().to_path_buf()],
             &pyproject_config,
             &ConfigArguments::default(),
             flags::Cache::Disabled,
@@ -284,7 +283,7 @@ mod test {
             .with_show_fix_status(true)
             .emit(
                 &mut output,
-                &diagnostics.messages,
+                &diagnostics.inner,
                 &EmitterContext::new(&FxHashMap::default()),
             )
             .unwrap();
