@@ -12,7 +12,7 @@ use crate::fix::edits::delete_comment;
 use crate::noqa::{
     Code, Directive, FileExemption, FileNoqaDirectives, NoqaDirectives, NoqaMapping,
 };
-use crate::registry::{AsRule, Rule, RuleSet};
+use crate::registry::Rule;
 use crate::rule_redirects::get_redirect_target;
 use crate::rules::pygrep_hooks;
 use crate::rules::ruff;
@@ -22,7 +22,7 @@ use crate::{Edit, Fix, Locator};
 
 use super::ast::LintContext;
 
-#[expect(clippy::too_many_arguments)]
+/// RUF100
 pub(crate) fn check_noqa(
     context: &mut LintContext,
     path: &Path,
@@ -30,7 +30,6 @@ pub(crate) fn check_noqa(
     comment_ranges: &CommentRanges,
     noqa_line_for: &NoqaMapping,
     analyze_directives: bool,
-    per_file_ignores: &RuleSet,
     settings: &LinterSettings,
 ) -> Vec<usize> {
     // Identify any codes that are globally exempted (within the current file).
@@ -47,14 +46,15 @@ pub(crate) fn check_noqa(
 
     // Remove any ignored diagnostics.
     'outer: for (index, diagnostic) in context.iter().enumerate() {
-        let rule = diagnostic.rule();
+        // Can't ignore syntax errors.
+        let Some(code) = diagnostic.noqa_code() else {
+            continue;
+        };
 
-        if matches!(rule, Rule::BlanketNOQA) {
+        if code == Rule::BlanketNOQA.noqa_code() {
             continue;
         }
 
-        let code = rule.noqa_code();
-
         match &exemption {
             FileExemption::All(_) => {
                 // If the file is exempted, ignore all diagnostics.
@@ -106,10 +106,9 @@ pub(crate) fn check_noqa(
 
     // Enforce that the noqa directive was actually used (RUF100), unless RUF100 was itself
     // suppressed.
-    if settings.rules.enabled(Rule::UnusedNOQA)
+    if context.is_rule_enabled(Rule::UnusedNOQA)
         && analyze_directives
         && !exemption.includes(Rule::UnusedNOQA)
-        && !per_file_ignores.contains(Rule::UnusedNOQA)
     {
         let directives = noqa_directives
             .lines()
@@ -148,7 +147,9 @@ pub(crate) fn check_noqa(
 
                         if seen_codes.insert(original_code) {
                             let is_code_used = if is_file_level {
-                                context.iter().any(|diag| diag.rule().noqa_code() == code)
+                                context
+                                    .iter()
+                                    .any(|diag| diag.noqa_code().is_some_and(|noqa| noqa == code))
                             } else {
                                 matches.iter().any(|match_| *match_ == code)
                             } || settings
@@ -159,7 +160,7 @@ pub(crate) fn check_noqa(
                             if is_code_used {
                                 valid_codes.push(original_code);
                             } else if let Ok(rule) = Rule::from_code(code) {
-                                if settings.rules.enabled(rule) {
+                                if context.is_rule_enabled(rule) {
                                     unmatched_codes.push(original_code);
                                 } else {
                                     disabled_codes.push(original_code);
@@ -229,18 +230,12 @@ pub(crate) fn check_noqa(
         }
     }
 
-    if settings.rules.enabled(Rule::RedirectedNOQA)
-        && !per_file_ignores.contains(Rule::RedirectedNOQA)
-        && !exemption.includes(Rule::RedirectedNOQA)
-    {
+    if context.is_rule_enabled(Rule::RedirectedNOQA) && !exemption.includes(Rule::RedirectedNOQA) {
         ruff::rules::redirected_noqa(context, &noqa_directives);
         ruff::rules::redirected_file_noqa(context, &file_noqa_directives);
     }
 
-    if settings.rules.enabled(Rule::BlanketNOQA)
-        && !per_file_ignores.contains(Rule::BlanketNOQA)
-        && !exemption.enumerates(Rule::BlanketNOQA)
-    {
+    if context.is_rule_enabled(Rule::BlanketNOQA) && !exemption.enumerates(Rule::BlanketNOQA) {
         pygrep_hooks::rules::blanket_noqa(
             context,
             &noqa_directives,
@@ -249,8 +244,7 @@ pub(crate) fn check_noqa(
         );
     }
 
-    if settings.rules.enabled(Rule::InvalidRuleCode)
-        && !per_file_ignores.contains(Rule::InvalidRuleCode)
+    if context.is_rule_enabled(Rule::InvalidRuleCode)
         && !exemption.enumerates(Rule::InvalidRuleCode)
     {
         ruff::rules::invalid_noqa_code(context, &noqa_directives, locator, &settings.external);
