@@ -11,7 +11,7 @@ mod tests {
 
     use crate::registry::{Linter, Rule};
     use crate::test::{test_path, test_snippet};
-    use crate::{assert_messages, settings};
+    use crate::{assert_diagnostics, settings};
 
     #[test_case(
         r#"
@@ -355,7 +355,7 @@ mod tests {
             contents,
             &settings::LinterSettings::for_rules(Linter::PandasVet.rules()),
         );
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
     }
 
     #[test_case(
@@ -370,7 +370,7 @@ mod tests {
             Path::new("pandas_vet").join(path).as_path(),
             &settings::LinterSettings::for_rule(rule_code),
         )?;
-        assert_messages!(snapshot, diagnostics);
+        assert_diagnostics!(snapshot, diagnostics);
         Ok(())
     }
 }
