@@ -1,17 +1,16 @@
-use ruff_diagnostics::AlwaysFixableViolation;
-use ruff_diagnostics::Diagnostic;
-use ruff_diagnostics::Edit;
-use ruff_diagnostics::Fix;
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_parser::TokenKind;
 use ruff_text_size::{Ranged, TextRange};
 
-use crate::checkers::logical_lines::LogicalLinesContext;
+use crate::AlwaysFixableViolation;
+use crate::Edit;
+use crate::Fix;
+use crate::checkers::ast::LintContext;
 
 use super::{LogicalLine, Whitespace};
 
 /// ## What it does
-/// Checks for the use of extraneous whitespace after "(".
+/// Checks for the use of extraneous whitespace after "(", "[" or "{".
 ///
 /// ## Why is this bad?
 /// [PEP 8] recommends the omission of whitespace in the following cases:
@@ -31,8 +30,8 @@ use super::{LogicalLine, Whitespace};
 /// ```
 ///
 /// [PEP 8]: https://peps.python.org/pep-0008/#pet-peeves
-#[violation]
-pub struct WhitespaceAfterOpenBracket {
+#[derive(ViolationMetadata)]
+pub(crate) struct WhitespaceAfterOpenBracket {
     symbol: char,
 }
 
@@ -50,7 +49,7 @@ impl AlwaysFixableViolation for WhitespaceAfterOpenBracket {
 }
 
 /// ## What it does
-/// Checks for the use of extraneous whitespace before ")".
+/// Checks for the use of extraneous whitespace before ")", "]" or "}".
 ///
 /// ## Why is this bad?
 /// [PEP 8] recommends the omission of whitespace in the following cases:
@@ -70,8 +69,8 @@ impl AlwaysFixableViolation for WhitespaceAfterOpenBracket {
 /// ```
 ///
 /// [PEP 8]: https://peps.python.org/pep-0008/#pet-peeves
-#[violation]
-pub struct WhitespaceBeforeCloseBracket {
+#[derive(ViolationMetadata)]
+pub(crate) struct WhitespaceBeforeCloseBracket {
     symbol: char,
 }
 
@@ -107,8 +106,8 @@ impl AlwaysFixableViolation for WhitespaceBeforeCloseBracket {
 /// ```
 ///
 /// [PEP 8]: https://peps.python.org/pep-0008/#pet-peeves
-#[violation]
-pub struct WhitespaceBeforePunctuation {
+#[derive(ViolationMetadata)]
+pub(crate) struct WhitespaceBeforePunctuation {
     symbol: char,
 }
 
@@ -126,7 +125,7 @@ impl AlwaysFixableViolation for WhitespaceBeforePunctuation {
 }
 
 /// E201, E202, E203
-pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLinesContext) {
+pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &LintContext) {
     let mut fstrings = 0u32;
     let mut brackets = vec![];
     let mut prev_token = None;
@@ -165,13 +164,13 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                 BracketOrPunctuation::OpenBracket(symbol) if symbol != '{' || fstrings == 0 => {
                     let (trailing, trailing_len) = line.trailing_whitespace(token);
                     if !matches!(trailing, Whitespace::None) {
-                        let mut diagnostic = Diagnostic::new(
+                        if let Some(mut diagnostic) = context.report_diagnostic_if_enabled(
                             WhitespaceAfterOpenBracket { symbol },
                             TextRange::at(token.end(), trailing_len),
-                        );
-                        diagnostic
-                            .set_fix(Fix::safe_edit(Edit::range_deletion(diagnostic.range())));
-                        context.push_diagnostic(diagnostic);
+                        ) {
+                            let range = diagnostic.range();
+                            diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(range)));
+                        }
                     }
                 }
                 BracketOrPunctuation::CloseBracket(symbol) if symbol != '}' || fstrings == 0 => {
@@ -179,13 +178,13 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                         if let (Whitespace::Single | Whitespace::Many | Whitespace::Tab, offset) =
                             line.leading_whitespace(token)
                         {
-                            let mut diagnostic = Diagnostic::new(
+                            if let Some(mut diagnostic) = context.report_diagnostic_if_enabled(
                                 WhitespaceBeforeCloseBracket { symbol },
                                 TextRange::at(token.start() - offset, offset),
-                            );
-                            diagnostic
-                                .set_fix(Fix::safe_edit(Edit::range_deletion(diagnostic.range())));
-                            context.push_diagnostic(diagnostic);
+                            ) {
+                                let range = diagnostic.range();
+                                diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(range)));
+                            }
                         }
                     }
                 }
@@ -205,14 +204,16 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                                 // If we're in the second half of a double colon, disallow
                                 // any whitespace (e.g., `foo[1: :2]` or `foo[1 : : 2]`).
                                 if matches!(prev_token, Some(TokenKind::Colon)) {
-                                    let mut diagnostic = Diagnostic::new(
-                                        WhitespaceBeforePunctuation { symbol },
-                                        TextRange::at(token.start() - offset, offset),
-                                    );
-                                    diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(
-                                        diagnostic.range(),
-                                    )));
-                                    context.push_diagnostic(diagnostic);
+                                    if let Some(mut diagnostic) = context
+                                        .report_diagnostic_if_enabled(
+                                            WhitespaceBeforePunctuation { symbol },
+                                            TextRange::at(token.start() - offset, offset),
+                                        )
+                                    {
+                                        let range = diagnostic.range();
+                                        diagnostic
+                                            .set_fix(Fix::safe_edit(Edit::range_deletion(range)));
+                                    }
                                 } else if iter.peek().is_some_and(|token| {
                                     matches!(token.kind(), TokenKind::Rsqb | TokenKind::Comma)
                                 }) {
@@ -220,14 +221,17 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                                     // Or `foo[index :, 2]`, but not `foo[index  :, 2]`.
                                     if let (Whitespace::Many | Whitespace::Tab, offset) = whitespace
                                     {
-                                        let mut diagnostic = Diagnostic::new(
-                                            WhitespaceBeforePunctuation { symbol },
-                                            TextRange::at(token.start() - offset, offset),
-                                        );
-                                        diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(
-                                            diagnostic.range(),
-                                        )));
-                                        context.push_diagnostic(diagnostic);
+                                        if let Some(mut diagnostic) = context
+                                            .report_diagnostic_if_enabled(
+                                                WhitespaceBeforePunctuation { symbol },
+                                                TextRange::at(token.start() - offset, offset),
+                                            )
+                                        {
+                                            let range = diagnostic.range();
+                                            diagnostic.set_fix(Fix::safe_edit(
+                                                Edit::range_deletion(range),
+                                            ));
+                                        }
                                     }
                                 } else if iter.peek().is_some_and(|token| {
                                     matches!(
@@ -245,15 +249,21 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                                     // whitespace before the colon and so should the fix
                                     if let (Whitespace::Many | Whitespace::Tab, offset) = whitespace
                                     {
-                                        let mut diagnostic = Diagnostic::new(
-                                            WhitespaceBeforePunctuation { symbol },
-                                            TextRange::at(token.start() - offset, offset),
-                                        );
-                                        diagnostic.set_fix(Fix::safe_edits(
-                                            Edit::range_deletion(diagnostic.range()),
-                                            [Edit::insertion(" ".into(), token.start() - offset)],
-                                        ));
-                                        context.push_diagnostic(diagnostic);
+                                        if let Some(mut diagnostic) = context
+                                            .report_diagnostic_if_enabled(
+                                                WhitespaceBeforePunctuation { symbol },
+                                                TextRange::at(token.start() - offset, offset),
+                                            )
+                                        {
+                                            let range = diagnostic.range();
+                                            diagnostic.set_fix(Fix::safe_edits(
+                                                Edit::range_deletion(range),
+                                                [Edit::insertion(
+                                                    " ".into(),
+                                                    token.start() - offset,
+                                                )],
+                                            ));
+                                        }
                                     }
                                 } else {
                                     // Allow, e.g., `foo[1:2]` or `foo[1 : 2]` or `foo[1 :: 2]`.
@@ -262,25 +272,34 @@ pub(crate) fn extraneous_whitespace(line: &LogicalLine, context: &mut LogicalLin
                                         .filter(|next| matches!(next.kind(), TokenKind::Colon))
                                         .unwrap_or(&token);
                                     if line.trailing_whitespace(token) != whitespace {
-                                        let mut diagnostic = Diagnostic::new(
-                                            WhitespaceBeforePunctuation { symbol },
-                                            TextRange::at(token.start() - offset, offset),
-                                        );
-                                        diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(
-                                            diagnostic.range(),
-                                        )));
-                                        context.push_diagnostic(diagnostic);
+                                        if let Some(mut diagnostic) = context
+                                            .report_diagnostic_if_enabled(
+                                                WhitespaceBeforePunctuation { symbol },
+                                                TextRange::at(token.start() - offset, offset),
+                                            )
+                                        {
+                                            let range = diagnostic.range();
+                                            diagnostic.set_fix(Fix::safe_edit(
+                                                Edit::range_deletion(range),
+                                            ));
+                                        }
                                     }
                                 }
                             } else {
-                                let mut diagnostic = Diagnostic::new(
+                                if fstrings > 0
+                                    && symbol == ':'
+                                    && matches!(prev_token, Some(TokenKind::Equal))
+                                {
+                                    // Avoid removing any whitespace for f-string debug expressions.
+                                    continue;
+                                }
+                                if let Some(mut diagnostic) = context.report_diagnostic_if_enabled(
                                     WhitespaceBeforePunctuation { symbol },
                                     TextRange::at(token.start() - offset, offset),
-                                );
-                                diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(
-                                    diagnostic.range(),
-                                )));
-                                context.push_diagnostic(diagnostic);
+                                ) {
+                                    let range = diagnostic.range();
+                                    diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(range)));
+                                }
                             }
                         }
                     }
