@@ -887,7 +887,7 @@ pub(crate) fn check_docstring(
         return;
     };
 
-    if checker.settings.pydoclint.ignore_one_line_docstrings && is_one_line(docstring) {
+    if checker.settings().pydoclint.ignore_one_line_docstrings && is_one_line(docstring) {
         return;
     }
 
@@ -915,11 +915,11 @@ pub(crate) fn check_docstring(
     };
 
     // DOC201
-    if checker.enabled(Rule::DocstringMissingReturns) {
+    if checker.is_rule_enabled(Rule::DocstringMissingReturns) {
         if should_document_returns(function_def)
             && !returns_documented(docstring, &docstring_sections, convention)
         {
-            let extra_property_decorators = checker.settings.pydocstyle.property_decorators();
+            let extra_property_decorators = checker.settings().pydocstyle.property_decorators();
             if !definition.is_property(extra_property_decorators, semantic) {
                 if !body_entries.returns.is_empty() {
                     match function_def.returns.as_deref() {
@@ -953,7 +953,7 @@ pub(crate) fn check_docstring(
     }
 
     // DOC402
-    if checker.enabled(Rule::DocstringMissingYields) {
+    if checker.is_rule_enabled(Rule::DocstringMissingYields) {
         if !yields_documented(docstring, &docstring_sections, convention) {
             if !body_entries.yields.is_empty() {
                 match function_def.returns.as_deref() {
@@ -974,7 +974,7 @@ pub(crate) fn check_docstring(
     }
 
     // DOC501
-    if checker.enabled(Rule::DocstringMissingException) {
+    if checker.is_rule_enabled(Rule::DocstringMissingException) {
         for body_raise in &body_entries.raised_exceptions {
             let Some(name) = body_raise.qualified_name.segments().last() else {
                 continue;
@@ -1006,7 +1006,7 @@ pub(crate) fn check_docstring(
     // document that it raises an exception without including the exception in the implementation.
     if !visibility::is_abstract(&function_def.decorator_list, semantic) {
         // DOC202
-        if checker.enabled(Rule::DocstringExtraneousReturns) {
+        if checker.is_rule_enabled(Rule::DocstringExtraneousReturns) {
             if docstring_sections.returns.is_some() {
                 if body_entries.returns.is_empty()
                     || body_entries.returns.iter().all(ReturnEntry::is_implicit)
@@ -1017,7 +1017,7 @@ pub(crate) fn check_docstring(
         }
 
         // DOC403
-        if checker.enabled(Rule::DocstringExtraneousYields) {
+        if checker.is_rule_enabled(Rule::DocstringExtraneousYields) {
             if docstring_sections.yields.is_some() {
                 if body_entries.yields.is_empty() {
                     checker.report_diagnostic(DocstringExtraneousYields, docstring.range());
@@ -1026,7 +1026,7 @@ pub(crate) fn check_docstring(
         }
 
         // DOC502
-        if checker.enabled(Rule::DocstringExtraneousException) {
+        if checker.is_rule_enabled(Rule::DocstringExtraneousException) {
             if let Some(docstring_raises) = docstring_sections.raises {
                 let mut extraneous_exceptions = Vec::new();
                 for docstring_raise in &docstring_raises.raised_exceptions {
