@@ -1,12 +1,13 @@
 use crate::Db;
 use crate::combine::Combine;
-use crate::glob::{ExcludeFilter, IncludeExcludeFilter, IncludeFilter};
+use crate::glob::{ExcludeFilter, IncludeExcludeFilter, IncludeFilter, PortableGlobKind};
 use crate::metadata::settings::{OverrideSettings, SrcSettings};
-use crate::metadata::value::{
-    RangedValue, RelativeExcludePattern, RelativeIncludePattern, RelativePathBuf, ValueSource,
-    ValueSourceGuard,
-};
 
+use super::settings::{Override, Settings, TerminalSettings};
+use crate::metadata::value::{
+    RangedValue, RelativeGlobPattern, RelativePathBuf, ValueSource, ValueSourceGuard,
+};
+use anyhow::Context;
 use ordermap::OrderMap;
 use ruff_db::RustDoc;
 use ruff_db::diagnostic::{
@@ -15,6 +16,7 @@ use ruff_db::diagnostic::{
 };
 use ruff_db::files::system_path_to_file;
 use ruff_db::system::{System, SystemPath, SystemPathBuf};
+use ruff_db::vendored::VendoredFileSystem;
 use ruff_macros::{Combine, OptionsMetadata, RustDoc};
 use ruff_options_metadata::{OptionSet, OptionsMetadata, Visit};
 use ruff_python_ast::PythonVersion;
@@ -28,12 +30,11 @@ use std::sync::Arc;
 use thiserror::Error;
 use ty_python_semantic::lint::{GetLintError, Level, LintSource, RuleSelection};
 use ty_python_semantic::{
-    ProgramSettings, PythonPath, PythonPlatform, PythonVersionFileSource, PythonVersionSource,
-    PythonVersionWithSource, SearchPathSettings, SysPrefixPathOrigin,
+    ProgramSettings, PythonEnvironment, PythonPlatform, PythonVersionFileSource,
+    PythonVersionSource, PythonVersionWithSource, SearchPathSettings, SearchPathValidationError,
+    SearchPaths, SitePackagesPaths, SysPrefixPathOrigin,
 };
 
-use super::settings::{Override, Settings, TerminalSettings};
-
 #[derive(
     Debug, Default, Clone, PartialEq, Eq, Combine, Serialize, Deserialize, OptionsMetadata,
 )]
@@ -105,45 +106,113 @@ impl Options {
         project_root: &SystemPath,
         project_name: &str,
         system: &dyn System,
-    ) -> ProgramSettings {
-        let python_version = self
-            .environment
-            .as_ref()
-            .and_then(|env| env.python_version.as_ref())
-            .map(|ranged_version| PythonVersionWithSource {
-                version: **ranged_version,
-                source: match ranged_version.source() {
-                    ValueSource::Cli => PythonVersionSource::Cli,
-                    ValueSource::File(path) => PythonVersionSource::ConfigFile(
-                        PythonVersionFileSource::new(path.clone(), ranged_version.range()),
-                    ),
-                },
-            });
-        let python_platform = self
-            .environment
-            .as_ref()
-            .and_then(|env| env.python_platform.as_deref().cloned())
+        vendored: &VendoredFileSystem,
+    ) -> anyhow::Result<ProgramSettings> {
+        let environment = self.environment.or_default();
+
+        let options_python_version =
+            environment
+                .python_version
+                .as_ref()
+                .map(|ranged_version| PythonVersionWithSource {
+                    version: **ranged_version,
+                    source: match ranged_version.source() {
+                        ValueSource::Cli => PythonVersionSource::Cli,
+                        ValueSource::File(path) => PythonVersionSource::ConfigFile(
+                            PythonVersionFileSource::new(path.clone(), ranged_version.range()),
+                        ),
+                    },
+                });
+
+        let python_platform = environment
+            .python_platform
+            .as_deref()
+            .cloned()
             .unwrap_or_else(|| {
                 let default = PythonPlatform::default();
                 tracing::info!("Defaulting to python-platform `{default}`");
                 default
             });
-        ProgramSettings {
+
+        let python_environment = if let Some(python_path) = environment.python.as_ref() {
+            let origin = match python_path.source() {
+                ValueSource::Cli => SysPrefixPathOrigin::PythonCliFlag,
+                ValueSource::File(path) => {
+                    SysPrefixPathOrigin::ConfigFileSetting(path.clone(), python_path.range())
+                }
+            };
+
+            Some(PythonEnvironment::new(
+                python_path.absolute(project_root, system),
+                origin,
+                system,
+            )?)
+        } else {
+            PythonEnvironment::discover(project_root, system)
+                .context("Failed to discover local Python environment")?
+        };
+
+        let site_packages_paths = if let Some(python_environment) = python_environment.as_ref() {
+            python_environment
+                .site_packages_paths(system)
+                .context("Failed to discover the site-packages directory")?
+        } else {
+            tracing::debug!("No virtual environment found");
+
+            SitePackagesPaths::default()
+        };
+
+        let python_version = options_python_version
+            .or_else(|| {
+                python_environment
+                    .as_ref()?
+                    .python_version_from_metadata()
+                    .cloned()
+            })
+            .or_else(|| site_packages_paths.python_version_from_layout())
+            .unwrap_or_default();
+
+        let search_paths = self.to_search_paths(
+            project_root,
+            project_name,
+            site_packages_paths,
+            system,
+            vendored,
+        )?;
+
+        tracing::info!(
+            "Python version: Python {python_version}, platform: {python_platform}",
+            python_version = python_version.version
+        );
+
+        Ok(ProgramSettings {
             python_version,
             python_platform,
-            search_paths: self.to_search_path_settings(project_root, project_name, system),
-        }
+            search_paths,
+        })
     }
 
-    fn to_search_path_settings(
+    fn to_search_paths(
         &self,
         project_root: &SystemPath,
         project_name: &str,
+        site_packages_paths: SitePackagesPaths,
         system: &dyn System,
-    ) -> SearchPathSettings {
-        let src_roots = if let Some(src_root) = self.src.as_ref().and_then(|src| src.root.as_ref())
+        vendored: &VendoredFileSystem,
+    ) -> Result<SearchPaths, SearchPathValidationError> {
+        let environment = self.environment.or_default();
+        let src = self.src.or_default();
+
+        #[allow(deprecated)]
+        let src_roots = if let Some(roots) = environment
+            .root
+            .as_deref()
+            .or_else(|| Some(std::slice::from_ref(src.root.as_ref()?)))
         {
-            vec![src_root.absolute(project_root, system)]
+            roots
+                .iter()
+                .map(|root| root.absolute(project_root, system))
+                .collect()
         } else {
             let src = project_root.join("src");
 
@@ -151,20 +220,20 @@ impl Options {
                 // Default to `src` and the project root if `src` exists and the root hasn't been specified.
                 // This corresponds to the `src-layout`
                 tracing::debug!(
-                    "Including `./src` in `src.root` because a `./src` directory exists"
+                    "Including `.` and `./src` in `environment.root` because a `./src` directory exists"
                 );
                 vec![project_root.to_path_buf(), src]
             } else if system.is_directory(&project_root.join(project_name).join(project_name)) {
                 // `src-layout` but when the folder isn't called `src` but has the same name as the project.
                 // For example, the "src" folder for `psycopg` is called `psycopg` and the python files are in `psycopg/psycopg/_adapters_map.py`
                 tracing::debug!(
-                    "Including `./{project_name}` in `src.root` because a `./{project_name}/{project_name}` directory exists"
+                    "Including `.` and `/{project_name}` in `environment.root` because a `./{project_name}/{project_name}` directory exists"
                 );
 
                 vec![project_root.to_path_buf(), project_root.join(project_name)]
             } else {
                 // Default to a [flat project structure](https://packaging.python.org/en/latest/discussions/src-layout-vs-flat-layout/).
-                tracing::debug!("Defaulting `src.root` to `.`");
+                tracing::debug!("Including `.` in `environment.root`");
                 vec![project_root.to_path_buf()]
             };
 
@@ -177,7 +246,7 @@ impl Options {
             {
                 // If the `tests` directory exists and is not a package, include it as a source root.
                 tracing::debug!(
-                    "Including `./tests` in `src.root` because a `./tests` directory exists"
+                    "Including `./tests` in `environment.root` because a `./tests` directory exists"
                 );
 
                 roots.push(tests_dir);
@@ -186,54 +255,23 @@ impl Options {
             roots
         };
 
-        let (extra_paths, python, typeshed) = self
-            .environment
-            .as_ref()
-            .map(|env| {
-                (
-                    env.extra_paths.clone(),
-                    env.python.clone(),
-                    env.typeshed.clone(),
-                )
-            })
-            .unwrap_or_default();
-
-        SearchPathSettings {
-            extra_paths: extra_paths
+        let settings = SearchPathSettings {
+            extra_paths: environment
+                .extra_paths
+                .as_deref()
                 .unwrap_or_default()
-                .into_iter()
+                .iter()
                 .map(|path| path.absolute(project_root, system))
                 .collect(),
             src_roots,
-            custom_typeshed: typeshed.map(|path| path.absolute(project_root, system)),
-            python_path: python
-                .map(|python_path| {
-                    let origin = match python_path.source() {
-                        ValueSource::Cli => SysPrefixPathOrigin::PythonCliFlag,
-                        ValueSource::File(path) => SysPrefixPathOrigin::ConfigFileSetting(
-                            path.clone(),
-                            python_path.range(),
-                        ),
-                    };
-                    PythonPath::sys_prefix(python_path.absolute(project_root, system), origin)
-                })
-                .or_else(|| {
-                    system.env_var("VIRTUAL_ENV").ok().map(|virtual_env| {
-                        PythonPath::sys_prefix(virtual_env, SysPrefixPathOrigin::VirtualEnvVar)
-                    })
-                })
-                .or_else(|| {
-                    system.env_var("CONDA_PREFIX").ok().map(|path| {
-                        PythonPath::sys_prefix(path, SysPrefixPathOrigin::CondaPrefixVar)
-                    })
-                })
-                .unwrap_or_else(|| {
-                    PythonPath::sys_prefix(
-                        project_root.to_path_buf(),
-                        SysPrefixPathOrigin::LocalVenv,
-                    )
-                }),
-        }
+            custom_typeshed: environment
+                .typeshed
+                .as_ref()
+                .map(|path| path.absolute(project_root, system)),
+            site_packages_paths: site_packages_paths.into_vec(),
+        };
+
+        settings.to_search_paths(system, vendored)
     }
 
     pub(crate) fn to_settings(
@@ -244,7 +282,7 @@ impl Options {
         let mut diagnostics = Vec::new();
         let rules = self.to_rule_selection(db, &mut diagnostics);
 
-        let terminal_options = self.terminal.clone().unwrap_or_default();
+        let terminal_options = self.terminal.or_default();
         let terminal = TerminalSettings {
             output_format: terminal_options
                 .output_format
@@ -254,11 +292,35 @@ impl Options {
             error_on_warning: terminal_options.error_on_warning.unwrap_or_default(),
         };
 
-        let src_options = if let Some(src) = self.src.as_ref() {
-            Cow::Borrowed(src)
-        } else {
-            Cow::Owned(SrcOptions::default())
-        };
+        let src_options = self.src.or_default();
+
+        #[allow(deprecated)]
+        if let Some(src_root) = src_options.root.as_ref() {
+            let mut diagnostic = OptionDiagnostic::new(
+                DiagnosticId::DeprecatedSetting,
+                "The `src.root` setting is deprecated. Use `environment.root` instead.".to_string(),
+                Severity::Warning,
+            );
+
+            if let Some(file) = src_root
+                .source()
+                .file()
+                .and_then(|path| system_path_to_file(db.upcast(), path).ok())
+            {
+                diagnostic = diagnostic.with_annotation(Some(Annotation::primary(
+                    Span::from(file).with_optional_range(src_root.range()),
+                )));
+            }
+
+            if self.environment.or_default().root.is_some() {
+                diagnostic = diagnostic.sub(SubDiagnostic::new(
+                    Severity::Info,
+                    "The `src.root` setting was ignored in favor of the `environment.root` setting",
+                ));
+            }
+
+            diagnostics.push(diagnostic);
+        }
 
         let src = src_options
             .to_settings(db, project_root, &mut diagnostics)
@@ -292,11 +354,7 @@ impl Options {
         db: &dyn Db,
         diagnostics: &mut Vec<OptionDiagnostic>,
     ) -> RuleSelection {
-        if let Some(rules) = self.rules.as_ref() {
-            rules.to_rule_selection(db, diagnostics)
-        } else {
-            RuleSelection::from_registry(db.lint_registry())
-        }
+        self.rules.or_default().to_rule_selection(db, diagnostics)
     }
 
     fn to_overrides_settings(
@@ -305,7 +363,7 @@ impl Options {
         project_root: &SystemPath,
         diagnostics: &mut Vec<OptionDiagnostic>,
     ) -> Result<Vec<Override>, Box<OptionDiagnostic>> {
-        let override_options = self.overrides.as_deref().unwrap_or_default();
+        let override_options = &**self.overrides.or_default();
 
         let mut overrides = Vec::with_capacity(override_options.len());
 
@@ -328,6 +386,29 @@ impl Options {
 #[serde(rename_all = "kebab-case", deny_unknown_fields)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 pub struct EnvironmentOptions {
+    /// The root paths of the project, used for finding first-party modules.
+    ///
+    /// Accepts a list of directory paths searched in priority order (first has highest priority).
+    ///
+    /// If left unspecified, ty will try to detect common project layouts and initialize `root` accordingly:
+    ///
+    /// * if a `./src` directory exists, include `.` and `./src` in the first party search path (src layout or flat)
+    /// * if a `./<project-name>/<project-name>` directory exists, include `.` and `./<project-name>` in the first party search path
+    /// * otherwise, default to `.` (flat layout)
+    ///
+    /// Besides, if a `./tests` directory exists and is not a package (i.e. it does not contain an `__init__.py` file),
+    /// it will also be included in the first party search path.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    #[option(
+        default = r#"null"#,
+        value_type = "list[str]",
+        example = r#"
+            # Multiple directories (priority order)
+            root = ["./src", "./lib", "./vendor"]
+        "#
+    )]
+    pub root: Option<Vec<RelativePathBuf>>,
+
     /// Specifies the version of Python that will be used to analyze the source code.
     /// The version should be specified as a string in the format `M.m` where `M` is the major version
     /// and `m` is the minor (e.g. `"3.0"` or `"3.6"`).
@@ -444,6 +525,7 @@ pub struct SrcOptions {
             root = "./app"
         "#
     )]
+    #[deprecated(note = "Use `environment.root` instead.")]
     pub root: Option<RelativePathBuf>,
 
     /// Whether to automatically exclude files that are ignored by `.ignore`,
@@ -475,7 +557,7 @@ pub struct SrcOptions {
     /// - `[abc]` matches any character inside the brackets. Character sequences can also specify ranges of characters, as ordered by Unicode,
     ///   so e.g. `[0-9]` specifies any character between `0` and `9` inclusive. An unclosed bracket is invalid.
     ///
-    /// Unlike `exclude`, all paths are anchored relative to the project root (`src` only
+    /// All paths are anchored relative to the project root (`src` only
     /// matches `<project_root>/src` and not `<project_root>/test/src`).
     ///
     /// `exclude` takes precedence over `include`.
@@ -490,14 +572,14 @@ pub struct SrcOptions {
             ]
         "#
     )]
-    pub include: Option<RangedValue<Vec<RelativeIncludePattern>>>,
+    pub include: Option<RangedValue<Vec<RelativeGlobPattern>>>,
 
     /// A list of file and directory patterns to exclude from type checking.
     ///
     /// Patterns follow a syntax similar to `.gitignore`:
     /// - `./src/` matches only a directory
     /// - `./src` matches both files and directories
-    /// - `src` matches files or directories named `src` anywhere in the tree (e.g. `./src` or `./tests/src`)
+    /// - `src` matches files or directories named `src`
     /// - `*` matches any (possibly empty) sequence of characters (except `/`).
     /// - `**` matches zero or more path components.
     ///   This sequence **must** form a single path component, so both `**a` and `b**` are invalid and will result in an error.
@@ -507,28 +589,32 @@ pub struct SrcOptions {
     ///   so e.g. `[0-9]` specifies any character between `0` and `9` inclusive. An unclosed bracket is invalid.
     /// - `!pattern` negates a pattern (undoes the exclusion of files that would otherwise be excluded)
     ///
-    /// By default, the following directories are excluded:
+    /// All paths are anchored relative to the project root (`src` only
+    /// matches `<project_root>/src` and not `<project_root>/test/src`).
+    /// To exclude any directory or file named `src`, use `**/src` instead.
     ///
-    /// - `.bzr`
-    /// - `.direnv`
-    /// - `.eggs`
-    /// - `.git`
-    /// - `.git-rewrite`
-    /// - `.hg`
-    /// - `.mypy_cache`
-    /// - `.nox`
-    /// - `.pants.d`
-    /// - `.pytype`
-    /// - `.ruff_cache`
-    /// - `.svn`
-    /// - `.tox`
-    /// - `.venv`
-    /// - `__pypackages__`
-    /// - `_build`
-    /// - `buck-out`
-    /// - `dist`
-    /// - `node_modules`
-    /// - `venv`
+    /// By default, ty excludes commonly ignored directories:
+    ///
+    /// - `**/.bzr/`
+    /// - `**/.direnv/`
+    /// - `**/.eggs/`
+    /// - `**/.git/`
+    /// - `**/.git-rewrite/`
+    /// - `**/.hg/`
+    /// - `**/.mypy_cache/`
+    /// - `**/.nox/`
+    /// - `**/.pants.d/`
+    /// - `**/.pytype/`
+    /// - `**/.ruff_cache/`
+    /// - `**/.svn/`
+    /// - `**/.tox/`
+    /// - `**/.venv/`
+    /// - `**/__pypackages__/`
+    /// - `**/_build/`
+    /// - `**/buck-out/`
+    /// - `**/dist/`
+    /// - `**/node_modules/`
+    /// - `**/venv/`
     ///
     /// You can override any default exclude by using a negated pattern. For example,
     /// to re-include `dist` use `exclude = ["!dist"]`
@@ -545,7 +631,7 @@ pub struct SrcOptions {
         "#
     )]
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub exclude: Option<RangedValue<Vec<RelativeExcludePattern>>>,
+    pub exclude: Option<RangedValue<Vec<RelativeGlobPattern>>>,
 }
 
 impl SrcOptions {
@@ -672,33 +758,33 @@ impl Rules {
 
 /// Default exclude patterns for src options.
 const DEFAULT_SRC_EXCLUDES: &[&str] = &[
-    ".bzr",
-    ".direnv",
-    ".eggs",
-    ".git",
-    ".git-rewrite",
-    ".hg",
-    ".mypy_cache",
-    ".nox",
-    ".pants.d",
-    ".pytype",
-    ".ruff_cache",
-    ".svn",
-    ".tox",
-    ".venv",
-    "__pypackages__",
-    "_build",
-    "buck-out",
-    "dist",
-    "node_modules",
-    "venv",
+    "**/.bzr/",
+    "**/.direnv/",
+    "**/.eggs/",
+    "**/.git/",
+    "**/.git-rewrite/",
+    "**/.hg/",
+    "**/.mypy_cache/",
+    "**/.nox/",
+    "**/.pants.d/",
+    "**/.pytype/",
+    "**/.ruff_cache/",
+    "**/.svn/",
+    "**/.tox/",
+    "**/.venv/",
+    "**/__pypackages__/",
+    "**/_build/",
+    "**/buck-out/",
+    "**/dist/",
+    "**/node_modules/",
+    "**/venv/",
 ];
 
 /// Helper function to build an include filter from patterns with proper error handling.
 fn build_include_filter(
     db: &dyn Db,
     project_root: &SystemPath,
-    include_patterns: Option<&RangedValue<Vec<RelativeIncludePattern>>>,
+    include_patterns: Option<&RangedValue<Vec<RelativeGlobPattern>>>,
     context: GlobFilterContext,
     diagnostics: &mut Vec<OptionDiagnostic>,
 ) -> Result<IncludeFilter, Box<OptionDiagnostic>> {
@@ -735,7 +821,7 @@ fn build_include_filter(
         }
 
         for pattern in include_patterns {
-            pattern.absolute(project_root, system)
+            pattern.absolute(project_root, system, PortableGlobKind::Include)
                 .and_then(|include| Ok(includes.add(&include)?))
                 .map_err(|err| {
                     let diagnostic = OptionDiagnostic::new(
@@ -773,7 +859,7 @@ fn build_include_filter(
     } else {
         includes
             .add(
-                &PortableGlobPattern::parse("**", false)
+                &PortableGlobPattern::parse("**", PortableGlobKind::Include)
                     .unwrap()
                     .into_absolute(""),
             )
@@ -797,7 +883,7 @@ fn build_include_filter(
 fn build_exclude_filter(
     db: &dyn Db,
     project_root: &SystemPath,
-    exclude_patterns: Option<&RangedValue<Vec<RelativeExcludePattern>>>,
+    exclude_patterns: Option<&RangedValue<Vec<RelativeGlobPattern>>>,
     default_patterns: &[&str],
     context: GlobFilterContext,
 ) -> Result<ExcludeFilter, Box<OptionDiagnostic>> {
@@ -807,7 +893,7 @@ fn build_exclude_filter(
     let mut excludes = ExcludeFilterBuilder::new();
 
     for pattern in default_patterns {
-        PortableGlobPattern::parse(pattern, true)
+        PortableGlobPattern::parse(pattern, PortableGlobKind::Exclude)
             .and_then(|exclude| Ok(excludes.add(&exclude.into_absolute(""))?))
             .unwrap_or_else(|err| {
                 panic!("Expected default exclude to be valid glob but adding it failed with: {err}")
@@ -817,7 +903,7 @@ fn build_exclude_filter(
     // Add user-specified excludes
     if let Some(exclude_patterns) = exclude_patterns {
         for exclude in exclude_patterns {
-            exclude.absolute(project_root, system)
+            exclude.absolute(project_root, system, PortableGlobKind::Exclude)
                 .and_then(|pattern| Ok(excludes.add(&pattern)?))
                 .map_err(|err| {
                     let diagnostic = OptionDiagnostic::new(
@@ -1001,7 +1087,7 @@ pub struct OverrideOptions {
             ]
         "#
     )]
-    pub include: Option<RangedValue<Vec<RelativeIncludePattern>>>,
+    pub include: Option<RangedValue<Vec<RelativeGlobPattern>>>,
 
     /// A list of file and directory patterns to exclude from this override.
     ///
@@ -1023,7 +1109,7 @@ pub struct OverrideOptions {
             ]
         "#
     )]
-    pub exclude: Option<RangedValue<Vec<RelativeExcludePattern>>>,
+    pub exclude: Option<RangedValue<Vec<RelativeGlobPattern>>>,
 
     /// Rule overrides for files matching the include/exclude patterns.
     ///
@@ -1053,8 +1139,10 @@ impl RangedValue<OverrideOptions> {
         global_rules: Option<&Rules>,
         diagnostics: &mut Vec<OptionDiagnostic>,
     ) -> Result<Option<Override>, Box<OptionDiagnostic>> {
+        let rules = self.rules.or_default();
+
         // First, warn about incorrect or useless overrides.
-        if self.rules.as_ref().is_none_or(Rules::is_empty) {
+        if rules.is_empty() {
             let mut diagnostic = OptionDiagnostic::new(
                 DiagnosticId::UselessOverridesSection,
                 "Useless `overrides` section".to_string(),
@@ -1171,11 +1259,11 @@ impl RangedValue<OverrideOptions> {
         let files = IncludeExcludeFilter::new(include, exclude);
 
         // Merge global rules with override rules, with override rules taking precedence
-        let merged_rules = self
-            .rules
-            .clone()
-            .combine(global_rules.cloned())
-            .expect("method to have early returned if rules is None");
+        let mut merged_rules = rules.into_owned();
+
+        if let Some(global_rules) = global_rules {
+            merged_rules = merged_rules.combine(global_rules.clone());
+        }
 
         // Convert merged rules to rule selection
         let rule_selection = merged_rules.to_rule_selection(db, diagnostics);
@@ -1388,3 +1476,23 @@ impl ProjectOptionsOverrides {
         }
     }
 }
+
+trait OrDefault {
+    type Target: ToOwned;
+
+    fn or_default(&self) -> Cow<'_, Self::Target>;
+}
+
+impl<T> OrDefault for Option<T>
+where
+    T: Default + ToOwned<Owned = T>,
+{
+    type Target = T;
+
+    fn or_default(&self) -> Cow<'_, Self::Target> {
+        match self {
+            Some(value) => Cow::Borrowed(value),
+            None => Cow::Owned(T::default()),
+        }
+    }
+}
