@@ -11,8 +11,11 @@
 use std::io;
 use std::num::NonZeroUsize;
 use std::ops::Deref;
+use std::str::FromStr;
 use std::{fmt, sync::Arc};
 
+use crate::{PythonVersionFileSource, PythonVersionSource, PythonVersionWithSource};
+use camino::Utf8Component;
 use indexmap::IndexSet;
 use ruff_annotate_snippets::{Level, Renderer, Snippet};
 use ruff_db::system::{System, SystemPath, SystemPathBuf};
@@ -21,8 +24,6 @@ use ruff_python_trivia::Cursor;
 use ruff_source_file::{LineIndex, OneIndexed, SourceCode};
 use ruff_text_size::{TextLen, TextRange};
 
-use crate::{PythonVersionFileSource, PythonVersionSource, PythonVersionWithSource};
-
 type SitePackagesDiscoveryResult<T> = Result<T, SitePackagesDiscoveryError>;
 
 /// An ordered, deduplicated set of `site-packages` search paths.
@@ -43,13 +44,9 @@ type SitePackagesDiscoveryResult<T> = Result<T, SitePackagesDiscoveryError>;
 /// *might* be added to the `SitePackagesPaths` twice, but we wouldn't
 /// want duplicates to appear in this set.
 #[derive(Debug, PartialEq, Eq, Default)]
-pub(crate) struct SitePackagesPaths(IndexSet<SystemPathBuf>);
+pub struct SitePackagesPaths(IndexSet<SystemPathBuf>);
 
 impl SitePackagesPaths {
-    pub(crate) fn len(&self) -> usize {
-        self.0.len()
-    }
-
     fn single(path: SystemPathBuf) -> Self {
         Self(IndexSet::from([path]))
     }
@@ -61,6 +58,54 @@ impl SitePackagesPaths {
     fn extend(&mut self, other: Self) {
         self.0.extend(other.0);
     }
+
+    /// Tries to detect the version from the layout of the `site-packages` directory.
+    pub fn python_version_from_layout(&self) -> Option<PythonVersionWithSource> {
+        if cfg!(windows) {
+            // The path to `site-packages` on Unix is
+            // `<sys.prefix>/lib/pythonX.Y/site-packages`,
+            // but on Windows it's `<sys.prefix>/Lib/site-packages`.
+            return None;
+        }
+
+        let primary_site_packages = self.0.first()?;
+
+        let mut site_packages_ancestor_components =
+            primary_site_packages.components().rev().skip(1).map(|c| {
+                // This should have all been validated in `site_packages.rs`
+                // when we resolved the search paths for the project.
+                debug_assert!(
+                    matches!(c, Utf8Component::Normal(_)),
+                    "Unexpected component in site-packages path `{c:?}` \
+                    (expected `site-packages` to be an absolute path with symlinks resolved, \
+                    located at `<sys.prefix>/lib/pythonX.Y/site-packages`)"
+                );
+
+                c.as_str()
+            });
+
+        let parent_component = site_packages_ancestor_components.next()?;
+
+        if site_packages_ancestor_components.next()? != "lib" {
+            return None;
+        }
+
+        let version = parent_component
+            .strip_prefix("python")
+            .or_else(|| parent_component.strip_prefix("pypy"))?
+            .trim_end_matches('t');
+
+        let version = PythonVersion::from_str(version).ok()?;
+        let source = PythonVersionSource::InstallationDirectoryLayout {
+            site_packages_parent_dir: Box::from(parent_component),
+        };
+
+        Some(PythonVersionWithSource { version, source })
+    }
+
+    pub fn into_vec(self) -> Vec<SystemPathBuf> {
+        self.0.into_iter().collect()
+    }
 }
 
 impl FromIterator<SystemPathBuf> for SitePackagesPaths {
@@ -85,13 +130,67 @@ impl PartialEq<&[SystemPathBuf]> for SitePackagesPaths {
 }
 
 #[derive(Debug)]
-pub(crate) enum PythonEnvironment {
+pub enum PythonEnvironment {
     Virtual(VirtualEnvironment),
     System(SystemEnvironment),
 }
 
 impl PythonEnvironment {
-    pub(crate) fn new(
+    pub fn discover(
+        project_root: &SystemPath,
+        system: &dyn System,
+    ) -> Result<Option<Self>, SitePackagesDiscoveryError> {
+        fn resolve_environment(
+            system: &dyn System,
+            path: &SystemPath,
+            origin: SysPrefixPathOrigin,
+        ) -> Result<PythonEnvironment, SitePackagesDiscoveryError> {
+            tracing::debug!("Resolving {origin}: {path}");
+            PythonEnvironment::new(path, origin, system)
+        }
+
+        if let Ok(virtual_env) = system.env_var("VIRTUAL_ENV") {
+            return resolve_environment(
+                system,
+                SystemPath::new(&virtual_env),
+                SysPrefixPathOrigin::VirtualEnvVar,
+            )
+            .map(Some);
+        }
+
+        if let Ok(conda_env) = system.env_var("CONDA_PREFIX") {
+            return resolve_environment(
+                system,
+                SystemPath::new(&conda_env),
+                SysPrefixPathOrigin::CondaPrefixVar,
+            )
+            .map(Some);
+        }
+
+        tracing::debug!("Discovering virtual environment in `{project_root}`");
+        let virtual_env_directory = project_root.join(".venv");
+
+        match PythonEnvironment::new(
+            &virtual_env_directory,
+            SysPrefixPathOrigin::LocalVenv,
+            system,
+        ) {
+            Ok(environment) => return Ok(Some(environment)),
+            Err(err) => {
+                if system.is_directory(&virtual_env_directory) {
+                    tracing::debug!(
+                        "Ignoring automatically detected virtual environment at `{}`: {}",
+                        &virtual_env_directory,
+                        err
+                    );
+                }
+            }
+        }
+
+        Ok(None)
+    }
+
+    pub fn new(
         path: impl AsRef<SystemPath>,
         origin: SysPrefixPathOrigin,
         system: &dyn System,
@@ -111,23 +210,17 @@ impl PythonEnvironment {
         }
     }
 
-    /// Returns the `site-packages` directories for this Python environment,
-    /// as well as the Python version that was used to create this environment
-    /// (the latter will only be available for virtual environments that specify
+    /// Returns the Python version that was used to create this environment
+    /// (will only be available for virtual environments that specify
     /// the metadata in their `pyvenv.cfg` files).
-    ///
-    /// See the documentation for [`site_packages_directory_from_sys_prefix`] for more details.
-    pub(crate) fn into_settings(
-        self,
-        system: &dyn System,
-    ) -> SitePackagesDiscoveryResult<(SitePackagesPaths, Option<PythonVersionWithSource>)> {
-        Ok(match self {
-            Self::Virtual(venv) => (venv.site_packages_directories(system)?, venv.version),
-            Self::System(env) => (env.site_packages_directories(system)?, None),
-        })
+    pub fn python_version_from_metadata(&self) -> Option<&PythonVersionWithSource> {
+        match self {
+            Self::Virtual(venv) => venv.version.as_ref(),
+            Self::System(_) => None,
+        }
     }
 
-    fn site_packages_directories(
+    pub fn site_packages_paths(
         &self,
         system: &dyn System,
     ) -> SitePackagesDiscoveryResult<SitePackagesPaths> {
@@ -173,7 +266,7 @@ impl PythonImplementation {
 /// The format of this file is not defined anywhere, and exactly which keys are present
 /// depends on the tool that was used to create the virtual environment.
 #[derive(Debug)]
-pub(crate) struct VirtualEnvironment {
+pub struct VirtualEnvironment {
     root_path: SysPrefixPath,
     base_executable_home_path: PythonHomePath,
     include_system_site_packages: bool,
@@ -322,7 +415,7 @@ impl VirtualEnvironment {
         );
 
         if let Some(parent_env_site_packages) = parent_environment.as_deref() {
-            match parent_env_site_packages.site_packages_directories(system) {
+            match parent_env_site_packages.site_packages_paths(system) {
                 Ok(parent_environment_site_packages) => {
                     site_packages_directories.extend(parent_environment_site_packages);
                 }
@@ -492,7 +585,7 @@ struct RawPyvenvCfg<'s> {
 /// This environment may or may not be one that is managed by the operating system itself, e.g.,
 /// this captures both Homebrew-installed Python versions and the bundled macOS Python installation.
 #[derive(Debug)]
-pub(crate) struct SystemEnvironment {
+pub struct SystemEnvironment {
     root_path: SysPrefixPath,
 }
 
@@ -532,7 +625,7 @@ impl SystemEnvironment {
 
 /// Enumeration of ways in which `site-packages` discovery can fail.
 #[derive(Debug)]
-pub(crate) enum SitePackagesDiscoveryError {
+pub enum SitePackagesDiscoveryError {
     /// `site-packages` discovery failed because the provided path couldn't be canonicalized.
     CanonicalizationError(SystemPathBuf, SysPrefixPathOrigin, io::Error),
 
@@ -698,7 +791,7 @@ fn display_error(
 
 /// The various ways in which parsing a `pyvenv.cfg` file could fail
 #[derive(Debug)]
-pub(crate) enum PyvenvCfgParseErrorKind {
+pub enum PyvenvCfgParseErrorKind {
     MalformedKeyValuePair { line_number: NonZeroUsize },
     NoHomeKey,
     InvalidHomeValue(io::Error),
@@ -853,7 +946,7 @@ fn site_packages_directory_from_sys_prefix(
 ///
 /// [`sys.prefix`]: https://docs.python.org/3/library/sys.html#sys.prefix
 #[derive(Debug, PartialEq, Eq, Clone)]
-pub(crate) struct SysPrefixPath {
+pub struct SysPrefixPath {
     inner: SystemPathBuf,
     origin: SysPrefixPathOrigin,
 }
@@ -864,10 +957,47 @@ impl SysPrefixPath {
         origin: SysPrefixPathOrigin,
         system: &dyn System,
     ) -> SitePackagesDiscoveryResult<Self> {
+        let sys_prefix = if !origin.must_point_directly_to_sys_prefix()
+            && system.is_file(unvalidated_path)
+            && unvalidated_path
+                .file_name()
+                .is_some_and(|name| name.starts_with("python"))
+        {
+            // It looks like they passed us a path to a Python executable, e.g. `.venv/bin/python3`.
+            // Try to figure out the `sys.prefix` value from the Python executable.
+            let sys_prefix = if cfg!(windows) {
+                // On Windows, the relative path to the Python executable from `sys.prefix`
+                // is different depending on whether it's a virtual environment or a system installation.
+                // System installations have their executable at `<sys.prefix>/python.exe`,
+                // whereas virtual environments have their executable at `<sys.prefix>/Scripts/python.exe`.
+                unvalidated_path.parent().and_then(|parent| {
+                    if parent.file_name() == Some("Scripts") {
+                        parent.parent()
+                    } else {
+                        Some(parent)
+                    }
+                })
+            } else {
+                // On Unix, `sys.prefix` is always the grandparent directory of the Python executable,
+                // regardless of whether it's a virtual environment or a system installation.
+                unvalidated_path.ancestors().nth(2)
+            };
+            let Some(sys_prefix) = sys_prefix else {
+                return Err(SitePackagesDiscoveryError::PathNotExecutableOrDirectory(
+                    unvalidated_path.to_path_buf(),
+                    origin,
+                    None,
+                ));
+            };
+            sys_prefix
+        } else {
+            unvalidated_path
+        };
+
         // It's important to resolve symlinks here rather than simply making the path absolute,
         // since system Python installations often only put symlinks in the "expected"
         // locations for `home` and `site-packages`
-        let canonicalized = match system.canonicalize_path(unvalidated_path) {
+        let sys_prefix = match system.canonicalize_path(sys_prefix) {
             Ok(path) => path,
             Err(io_err) => {
                 let unvalidated_path = unvalidated_path.to_path_buf();
@@ -888,69 +1018,19 @@ impl SysPrefixPath {
             }
         };
 
-        if origin.must_point_directly_to_sys_prefix() {
-            return if system.is_directory(&canonicalized) {
-                Ok(Self {
-                    inner: canonicalized,
-                    origin,
-                })
-            } else {
-                Err(SitePackagesDiscoveryError::PathNotExecutableOrDirectory(
-                    unvalidated_path.to_path_buf(),
-                    origin,
-                    None,
-                ))
-            };
-        }
-
-        let sys_prefix = if system.is_file(&canonicalized)
-            && canonicalized
-                .file_name()
-                .is_some_and(|name| name.starts_with("python"))
-        {
-            // It looks like they passed us a path to a Python executable, e.g. `.venv/bin/python3`.
-            // Try to figure out the `sys.prefix` value from the Python executable.
-            let sys_prefix = if cfg!(windows) {
-                // On Windows, the relative path to the Python executable from `sys.prefix`
-                // is different depending on whether it's a virtual environment or a system installation.
-                // System installations have their executable at `<sys.prefix>/python.exe`,
-                // whereas virtual environments have their executable at `<sys.prefix>/Scripts/python.exe`.
-                canonicalized.parent().and_then(|parent| {
-                    if parent.file_name() == Some("Scripts") {
-                        parent.parent()
-                    } else {
-                        Some(parent)
-                    }
-                })
-            } else {
-                // On Unix, `sys.prefix` is always the grandparent directory of the Python executable,
-                // regardless of whether it's a virtual environment or a system installation.
-                canonicalized.ancestors().nth(2)
-            };
-            let Some(sys_prefix) = sys_prefix else {
-                return Err(SitePackagesDiscoveryError::PathNotExecutableOrDirectory(
-                    unvalidated_path.to_path_buf(),
-                    origin,
-                    None,
-                ));
-            };
-            sys_prefix.to_path_buf()
-        } else if system.is_directory(&canonicalized) {
-            canonicalized
-        } else {
+        if !system.is_directory(&sys_prefix) {
             return Err(SitePackagesDiscoveryError::PathNotExecutableOrDirectory(
                 unvalidated_path.to_path_buf(),
                 origin,
                 None,
             ));
-        };
+        }
 
         Ok(Self {
             inner: sys_prefix,
             origin,
         })
     }
-
     fn from_executable_home_path(path: &PythonHomePath) -> Option<Self> {
         // No need to check whether `path.parent()` is a directory:
         // the parent of a canonicalised path that is known to exist
@@ -1612,7 +1692,7 @@ mod tests {
         // directory
         let env =
             PythonEnvironment::new("/env", SysPrefixPathOrigin::PythonCliFlag, &system).unwrap();
-        let site_packages = env.site_packages_directories(&system);
+        let site_packages = env.site_packages_paths(&system);
         if cfg!(unix) {
             assert!(
                 matches!(
@@ -1651,7 +1731,7 @@ mod tests {
         // Environment creation succeeds, but site-packages retrieval fails
         let env =
             PythonEnvironment::new("/env", SysPrefixPathOrigin::PythonCliFlag, &system).unwrap();
-        let site_packages = env.site_packages_directories(&system);
+        let site_packages = env.site_packages_paths(&system);
         assert!(
             matches!(
                 site_packages,
